////////////////////////////////////////////////////////////////////////////////////////////
// LEDMulti.c - plik gwny programu obsugi multipleksowanego wywietlacza LED
//     przeznaczenie: "Programowanie procesorw w jzyku C (EdW)"
//
// Autor: Radosaw Koppel          Kompilator: WinAVR 20050214
////////////////////////////////////////////////////////////////////////////////////////////
// Komenta dodatkowy: 
//   Obsuga mulitpleksowanego wywietlacza LED za pomoc przerwa. Dwukrotnie zwikszenie 
//  czstotliwoci wywoywania przerwania przez wpisywanie do licznika pocztkowej wartoci. 
//   Umieszczenie tablicy inicjacji kolumn wywietlacza w pamici programu. 
//   Umieszczenie danych w pamici programu za pomoc odpowiedniej zmiennej. 
//
/////////////////////////////////////////////////////////////////////////////////////////////
#include <avr\io.h>
#include <inttypes.h>
#include <avr\signal.h>
#include <avr\interrupt.h>
#include <avr\pgmspace.h>

// Definicje wyprowadze
#define LED_A 0
#define LED_B 1
#define LED_C 2
#define LED_D 3
#define LED_E 4
#define LED_F 5
#define LED_G 6
#define LED_DP 7
#define LEDPORT PORTB
#define LEDDDR DDRB
#define COM1 6
#define COM2 5
#define COM3 4
#define COM4 3
#define COMPORT PORTD
#define COMDDR DDRD

uint8_t g_AktWyswietlacz = 0; 
uint8_t g_DaneWyswietlacza[4]; 
uint8_t g_DaneCom[4] PROGMEM = 
	{~(1<<COM1), ~(1<<COM2), ~(1<<COM3), ~(1<<COM4)}; 

int main(void)
{
	/////////////////////////////
	// inicjacja
	LEDDDR = 0xff; 
	COMDDR = 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4; 
	// Timer0
	TCCR0 = 1<<CS01|1<<CS00; 
	TIMSK = 1<<TOIE0; 
	// Globalne zezwolenie na przerwania
	sei(); 
	// koniec inicjacji
	/////////////////////////////
	
	// Wpisanie do tablicy prbnych wartoci
	g_DaneWyswietlacza[0] = ~(1<<LED_B | 1<<LED_C); 
	g_DaneWyswietlacza[1] = ~(1<<LED_A | 1<<LED_B | 1<<LED_D | 1<<LED_E | 1<<LED_G); 
	g_DaneWyswietlacza[2] = ~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_G); 
	g_DaneWyswietlacza[3] = ~(1<<LED_B | 1<<LED_C | 1<<LED_F | 1<<LED_G); 
	
	for(;;)
	{
	}

	return 0; 
}

//_________________________________________________________________________________
// Obsuga przerwa
SIGNAL(SIG_OVERFLOW0)
{
	// Wpisanie do licznika pocztkowej wartoci
	TCNT0 = 128; 
	// Wygaszenie wywietlaczy
	COMPORT |= 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4; 
	// Wysanie odpowiedniej danej
	LEDPORT = g_DaneWyswietlacza[g_AktWyswietlacz]; 
	// Wczenie odpowiedniego wywietlacza
	COMPORT &= pgm_read_byte(&g_DaneCom[g_AktWyswietlacz]); 
	// Zwikszenie stanu zmiennej wskazujcej na obsugiwany wywietlacz
	++g_AktWyswietlacz; 
	if(g_AktWyswietlacz > 3)
		g_AktWyswietlacz = 0; 
}
