////////////////////////////////////////////////////////////////////////////////////////////
// LEDMulti.c - plik gwny programu obsugi multipleksowanego wywietlacza LED
//     przeznaczenie: "Programowanie procesorw w jzyku C (EdW)"
//
// Autor: Radosaw Koppel          Kompilator: WinAVR 20050214
////////////////////////////////////////////////////////////////////////////////////////////
// Komenta dodatkowy: 
//   Obsuga mulitpleksowanego wywietlacza LED za pomoc przerwa. Dwukrotnie zwikszenie 
//  czstotliwoci wywoywania przerwania przez wpisywanie do licznika pocztkowej wartoci. 
//   Umieszczenie tablicy inicjacji kolumn wywietlacza w pamici programu. 
//   Umieszczenie danych w pamici programu za pomoc odpowiedniej zmiennej. 
// 
// Wywietlanie staej wartoci z pamieci programu za pomoc funkcji WyswietlHex
// Przykad wykorzystania 
//
/////////////////////////////////////////////////////////////////////////////////////////////
#include <avr\io.h>
#include <inttypes.h>
#include <avr\signal.h>
#include <avr\interrupt.h>
#include <avr\pgmspace.h>
#include <avr\delay.h>

// Definicje wyprowadze
#define LED_A 0
#define LED_B 1
#define LED_C 2
#define LED_D 3
#define LED_E 4
#define LED_F 5
#define LED_G 6
#define LED_DP 7
#define LEDPORT PORTB
#define LEDDDR DDRB
#define COM1 6
#define COM2 5
#define COM3 4
#define COM4 3
#define COMPORT PORTD
#define COMDDR DDRD

uint8_t g_AktWyswietlacz = 0; 
uint8_t g_DaneWyswietlacza[4]; 
prog_uint8_t g_DaneCom[4] = 
	{~(1<<COM1), ~(1<<COM2), ~(1<<COM3), ~(1<<COM4)}; 

prog_uint8_t g_WzorCyfr[16] = 
	{
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_F), // 0	
		~(1<<LED_B | 1<<LED_C), // 1
		~(1<<LED_A | 1<<LED_B | 1<<LED_G | 1<<LED_E | 1<<LED_D), // 2
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_G), // 3
		~(1<<LED_B | 1<<LED_C | 1<<LED_G | 1<<LED_F), // 4
		~(1<<LED_A | 1<<LED_F | 1<<LED_G | 1<<LED_C | 1<<LED_D), // 5
		~(1<<LED_A | 1<<LED_F | 1<<LED_E | 1<<LED_D | 1<<LED_C |1<< LED_G), // 6
		~(1<<LED_A | 1<<LED_B | 1<<LED_C), // 7
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_F | 1<<LED_G), // 8
		~(1<<LED_G | 1<<LED_F | 1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_D), // 9
		~(1<<LED_A | 1<<LED_B | 1<<LED_C | 1<<LED_E | 1<<LED_F | 1<<LED_G), // A
		~(1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_F | 1<<LED_G), // B
		~(1<<LED_A | 1<<LED_D | 1<<LED_E | 1<<LED_F), // C
		~(1<<LED_B | 1<<LED_C | 1<<LED_D | 1<<LED_E | 1<<LED_G), // D
		~(1<<LED_A | 1<<LED_D | 1<<LED_E | 1<<LED_F | 1<<LED_G), // E
		~(1<<LED_A | 1<<LED_E | 1<<LED_F | 1<<LED_G) // F
	}; 

prog_uint16_t g_Stala = 0xcdef; 


// Funkcja wywietla w kodzie heksadecymalnym podan liczb
void WyswietlHex(uint16_t var)
{
	g_DaneWyswietlacza[0] = pgm_read_byte(&g_WzorCyfr[var>>(3*4)]); 
	g_DaneWyswietlacza[1] = pgm_read_byte(&g_WzorCyfr[(var>>(2*4))&0xf]);
	g_DaneWyswietlacza[2] = pgm_read_byte(&g_WzorCyfr[(var>>(1*4))&0xf]);
	g_DaneWyswietlacza[3] = pgm_read_byte(&g_WzorCyfr[var&0xf]);
}

// Start
int main(void)
{
	/////////////////////////////
	// inicjacja
	LEDDDR = 0xff; 
	COMDDR = 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4; 
	// Timer0
	TCCR0 = 1<<CS01|1<<CS00; 
	TIMSK = 1<<TOIE0; 
	// Globalne zezwolenie na przerwania
	sei(); 
	// koniec inicjacji
	/////////////////////////////
	
	WyswietlHex(pgm_read_word(&g_Stala)); 
	for(;;)
	{
	}

	return 0; 
}

//_________________________________________________________________________________
// Obsuga przerwa
SIGNAL(SIG_OVERFLOW0)
{
	// Wpisanie do licznika pocztkowej wartoci
	TCNT0 = 128; 
	// Wygaszenie wywietlaczy
	COMPORT |= 1<<COM1 | 1<<COM2 | 1<<COM3 | 1<<COM4; 
	// Wysanie odpowiedniej danej
	LEDPORT = g_DaneWyswietlacza[g_AktWyswietlacz]; 
	// Wczenie odpowiedniego wywietlacza
	COMPORT &= pgm_read_byte(&g_DaneCom[g_AktWyswietlacz]); 
	// Zwikszenie stanu zmiennej wskazujcej na obsugiwany wywietlacz
	++g_AktWyswietlacz; 
	if(g_AktWyswietlacz > 3)
		g_AktWyswietlacz = 0; 
}
